'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var uuid = require('uuid');
var lodash = require('lodash');
var deepmerge = _interopDefault(require('deepmerge'));

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

if (!Promise.any) {
    Promise.any = function any(promises) {
        if (!Array.isArray(promises) || promises.length === 0) {
            return Promise.reject('any is empty');
        }
        return Promise.all(
        /** @ts-ignore **/
        promises.map(function (promise) {
            return promise.then(function (val) {
                throw val;
            }, function (reason) { return reason; });
        })).then(function (reasons) {
            throw reasons;
        }, function (firstResolved) { return firstResolved; });
    };
}

var version = "1.0.0";

var NAME = 'jbz-film-logic';

var hasConfig = function () {
    return window.$jbzLogicConfig !== undefined;
};
var getConfig = function () {
    if (!hasConfig()) {
        console.error("\u672A\u5B89\u88C5 " + NAME + " \u63D2\u4EF6\uFF0C\u8BF7\u5148\u5B89\u88C5\u672C\u63D2\u4EF6");
        return null;
    }
    return window.$jbzLogicConfig;
};
var NETWORK_CONFIG = null;
var getNetwork = function (networkName) {
    if (NETWORK_CONFIG) {
        if (NETWORK_CONFIG.hasOwnProperty(networkName)) {
            return NETWORK_CONFIG[networkName];
        }
    }
    var jbzLogicConfig = getConfig();
    if (!jbzLogicConfig) {
        return;
    }
    var networkConfig = jbzLogicConfig.networkConfig;
    if (!networkConfig) {
        console.error('没有找到 networkConfig 配置项');
        return null;
    }
    NETWORK_CONFIG = networkConfig;
    if (networkConfig.hasOwnProperty(networkName)) {
        return networkConfig[networkName];
    }
    console.error("\u6CA1\u6709\u627E\u5230\u7F51\u7EDC\u914D\u7F6E " + networkName);
    return null;
};
var getMapConfig = function () {
    var jbzLogicConfig = getConfig();
    if (!jbzLogicConfig) {
        return;
    }
    var geoConfig = jbzLogicConfig.geoConfig;
    if (!geoConfig) {
        console.error('没有找到 geoConfig 配置项');
        return null;
    }
    var mapPlatforms = geoConfig.mapPlatforms;
    if (!(Array.isArray(mapPlatforms) || typeof mapPlatforms === 'string')) {
        console.error("mapFrameworks \u914D\u7F6E\u9879\u683C\u5F0F\u9519\u8BEF");
        return null;
    }
    return geoConfig;
};

/**
 * logic类型 目前仅使用 MIXIN，预留字段
 */
var LogicType;
(function (LogicType) {
    LogicType["MIXIN"] = "mixin";
})(LogicType || (LogicType = {}));
/**
 * 请求序列发送方式
 */
var RequestSequenceType;
(function (RequestSequenceType) {
    /**
     * 一个接一个的发送, 当之前的请求收到数据后发送第二个请求
     */
    RequestSequenceType["ONEBYONE"] = "onebyone";
    /**
     * 同时发出请求，不限制数据返回顺序
     */
    RequestSequenceType["CONCURRENCY"] = "concurrency";
    /**
     * 同事发送请求，当收到所有请求的数据才返回
     */
    RequestSequenceType["ALLSETTLED"] = "allsettled";
})(RequestSequenceType || (RequestSequenceType = {}));
/**
 * 单个请求的发送状态
 */
var RequestStatus;
(function (RequestStatus) {
    /**
     * 初始化
     */
    RequestStatus["INIT"] = "init";
    /**
     * 已经发送但没有返回
     */
    RequestStatus["HOLD"] = "hold";
    /**
     * 已经收到数据
     */
    RequestStatus["FINISH"] = "finish";
})(RequestStatus || (RequestStatus = {}));
var LocatorType;
(function (LocatorType) {
    LocatorType["BROWSER"] = "browser";
    LocatorType["NATIVE"] = "native";
    LocatorType["BAIDU"] = "baidu";
    LocatorType["GAODE"] = "gaode";
    LocatorType["TENCENT"] = "tencent";
})(LocatorType || (LocatorType = {}));

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
function isFunction(x) {
    return typeof x === 'function';
}

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
var _enable_super_gross_mode_that_will_cause_bad_things = false;
var config = {
    Promise: undefined,
    set useDeprecatedSynchronousErrorHandling(value) {
        if (value) {
            var error = /*@__PURE__*/ new Error();
            /*@__PURE__*/ console.warn('DEPRECATED! RxJS was set to use deprecated synchronous error handling behavior by code at: \n' + error.stack);
        }
        _enable_super_gross_mode_that_will_cause_bad_things = value;
    },
    get useDeprecatedSynchronousErrorHandling() {
        return _enable_super_gross_mode_that_will_cause_bad_things;
    },
};

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
function hostReportError(err) {
    setTimeout(function () { throw err; }, 0);
}

/** PURE_IMPORTS_START _config,_util_hostReportError PURE_IMPORTS_END */
var empty = {
    closed: true,
    next: function (value) { },
    error: function (err) {
        if (config.useDeprecatedSynchronousErrorHandling) {
            throw err;
        }
        else {
            hostReportError(err);
        }
    },
    complete: function () { }
};

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
var isArray = /*@__PURE__*/ (function () { return Array.isArray || (function (x) { return x && typeof x.length === 'number'; }); })();

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
function isObject(x) {
    return x !== null && typeof x === 'object';
}

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
var UnsubscriptionErrorImpl = /*@__PURE__*/ (function () {
    function UnsubscriptionErrorImpl(errors) {
        Error.call(this);
        this.message = errors ?
            errors.length + " errors occurred during unsubscription:\n" + errors.map(function (err, i) { return i + 1 + ") " + err.toString(); }).join('\n  ') : '';
        this.name = 'UnsubscriptionError';
        this.errors = errors;
        return this;
    }
    UnsubscriptionErrorImpl.prototype = /*@__PURE__*/ Object.create(Error.prototype);
    return UnsubscriptionErrorImpl;
})();
var UnsubscriptionError = UnsubscriptionErrorImpl;

/** PURE_IMPORTS_START _util_isArray,_util_isObject,_util_isFunction,_util_UnsubscriptionError PURE_IMPORTS_END */
var Subscription = /*@__PURE__*/ (function () {
    function Subscription(unsubscribe) {
        this.closed = false;
        this._parentOrParents = null;
        this._subscriptions = null;
        if (unsubscribe) {
            this._unsubscribe = unsubscribe;
        }
    }
    Subscription.prototype.unsubscribe = function () {
        var errors;
        if (this.closed) {
            return;
        }
        var _a = this, _parentOrParents = _a._parentOrParents, _unsubscribe = _a._unsubscribe, _subscriptions = _a._subscriptions;
        this.closed = true;
        this._parentOrParents = null;
        this._subscriptions = null;
        if (_parentOrParents instanceof Subscription) {
            _parentOrParents.remove(this);
        }
        else if (_parentOrParents !== null) {
            for (var index = 0; index < _parentOrParents.length; ++index) {
                var parent_1 = _parentOrParents[index];
                parent_1.remove(this);
            }
        }
        if (isFunction(_unsubscribe)) {
            try {
                _unsubscribe.call(this);
            }
            catch (e) {
                errors = e instanceof UnsubscriptionError ? flattenUnsubscriptionErrors(e.errors) : [e];
            }
        }
        if (isArray(_subscriptions)) {
            var index = -1;
            var len = _subscriptions.length;
            while (++index < len) {
                var sub = _subscriptions[index];
                if (isObject(sub)) {
                    try {
                        sub.unsubscribe();
                    }
                    catch (e) {
                        errors = errors || [];
                        if (e instanceof UnsubscriptionError) {
                            errors = errors.concat(flattenUnsubscriptionErrors(e.errors));
                        }
                        else {
                            errors.push(e);
                        }
                    }
                }
            }
        }
        if (errors) {
            throw new UnsubscriptionError(errors);
        }
    };
    Subscription.prototype.add = function (teardown) {
        var subscription = teardown;
        if (!teardown) {
            return Subscription.EMPTY;
        }
        switch (typeof teardown) {
            case 'function':
                subscription = new Subscription(teardown);
            case 'object':
                if (subscription === this || subscription.closed || typeof subscription.unsubscribe !== 'function') {
                    return subscription;
                }
                else if (this.closed) {
                    subscription.unsubscribe();
                    return subscription;
                }
                else if (!(subscription instanceof Subscription)) {
                    var tmp = subscription;
                    subscription = new Subscription();
                    subscription._subscriptions = [tmp];
                }
                break;
            default: {
                throw new Error('unrecognized teardown ' + teardown + ' added to Subscription.');
            }
        }
        var _parentOrParents = subscription._parentOrParents;
        if (_parentOrParents === null) {
            subscription._parentOrParents = this;
        }
        else if (_parentOrParents instanceof Subscription) {
            if (_parentOrParents === this) {
                return subscription;
            }
            subscription._parentOrParents = [_parentOrParents, this];
        }
        else if (_parentOrParents.indexOf(this) === -1) {
            _parentOrParents.push(this);
        }
        else {
            return subscription;
        }
        var subscriptions = this._subscriptions;
        if (subscriptions === null) {
            this._subscriptions = [subscription];
        }
        else {
            subscriptions.push(subscription);
        }
        return subscription;
    };
    Subscription.prototype.remove = function (subscription) {
        var subscriptions = this._subscriptions;
        if (subscriptions) {
            var subscriptionIndex = subscriptions.indexOf(subscription);
            if (subscriptionIndex !== -1) {
                subscriptions.splice(subscriptionIndex, 1);
            }
        }
    };
    Subscription.EMPTY = (function (empty) {
        empty.closed = true;
        return empty;
    }(new Subscription()));
    return Subscription;
}());
function flattenUnsubscriptionErrors(errors) {
    return errors.reduce(function (errs, err) { return errs.concat((err instanceof UnsubscriptionError) ? err.errors : err); }, []);
}

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
var rxSubscriber = /*@__PURE__*/ (function () {
    return typeof Symbol === 'function'
        ? /*@__PURE__*/ Symbol('rxSubscriber')
        : '@@rxSubscriber_' + /*@__PURE__*/ Math.random();
})();

/** PURE_IMPORTS_START tslib,_util_isFunction,_Observer,_Subscription,_internal_symbol_rxSubscriber,_config,_util_hostReportError PURE_IMPORTS_END */
var Subscriber = /*@__PURE__*/ (function (_super) {
    __extends(Subscriber, _super);
    function Subscriber(destinationOrNext, error, complete) {
        var _this = _super.call(this) || this;
        _this.syncErrorValue = null;
        _this.syncErrorThrown = false;
        _this.syncErrorThrowable = false;
        _this.isStopped = false;
        switch (arguments.length) {
            case 0:
                _this.destination = empty;
                break;
            case 1:
                if (!destinationOrNext) {
                    _this.destination = empty;
                    break;
                }
                if (typeof destinationOrNext === 'object') {
                    if (destinationOrNext instanceof Subscriber) {
                        _this.syncErrorThrowable = destinationOrNext.syncErrorThrowable;
                        _this.destination = destinationOrNext;
                        destinationOrNext.add(_this);
                    }
                    else {
                        _this.syncErrorThrowable = true;
                        _this.destination = new SafeSubscriber(_this, destinationOrNext);
                    }
                    break;
                }
            default:
                _this.syncErrorThrowable = true;
                _this.destination = new SafeSubscriber(_this, destinationOrNext, error, complete);
                break;
        }
        return _this;
    }
    Subscriber.prototype[rxSubscriber] = function () { return this; };
    Subscriber.create = function (next, error, complete) {
        var subscriber = new Subscriber(next, error, complete);
        subscriber.syncErrorThrowable = false;
        return subscriber;
    };
    Subscriber.prototype.next = function (value) {
        if (!this.isStopped) {
            this._next(value);
        }
    };
    Subscriber.prototype.error = function (err) {
        if (!this.isStopped) {
            this.isStopped = true;
            this._error(err);
        }
    };
    Subscriber.prototype.complete = function () {
        if (!this.isStopped) {
            this.isStopped = true;
            this._complete();
        }
    };
    Subscriber.prototype.unsubscribe = function () {
        if (this.closed) {
            return;
        }
        this.isStopped = true;
        _super.prototype.unsubscribe.call(this);
    };
    Subscriber.prototype._next = function (value) {
        this.destination.next(value);
    };
    Subscriber.prototype._error = function (err) {
        this.destination.error(err);
        this.unsubscribe();
    };
    Subscriber.prototype._complete = function () {
        this.destination.complete();
        this.unsubscribe();
    };
    Subscriber.prototype._unsubscribeAndRecycle = function () {
        var _parentOrParents = this._parentOrParents;
        this._parentOrParents = null;
        this.unsubscribe();
        this.closed = false;
        this.isStopped = false;
        this._parentOrParents = _parentOrParents;
        return this;
    };
    return Subscriber;
}(Subscription));
var SafeSubscriber = /*@__PURE__*/ (function (_super) {
    __extends(SafeSubscriber, _super);
    function SafeSubscriber(_parentSubscriber, observerOrNext, error, complete) {
        var _this = _super.call(this) || this;
        _this._parentSubscriber = _parentSubscriber;
        var next;
        var context = _this;
        if (isFunction(observerOrNext)) {
            next = observerOrNext;
        }
        else if (observerOrNext) {
            next = observerOrNext.next;
            error = observerOrNext.error;
            complete = observerOrNext.complete;
            if (observerOrNext !== empty) {
                context = Object.create(observerOrNext);
                if (isFunction(context.unsubscribe)) {
                    _this.add(context.unsubscribe.bind(context));
                }
                context.unsubscribe = _this.unsubscribe.bind(_this);
            }
        }
        _this._context = context;
        _this._next = next;
        _this._error = error;
        _this._complete = complete;
        return _this;
    }
    SafeSubscriber.prototype.next = function (value) {
        if (!this.isStopped && this._next) {
            var _parentSubscriber = this._parentSubscriber;
            if (!config.useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                this.__tryOrUnsub(this._next, value);
            }
            else if (this.__tryOrSetError(_parentSubscriber, this._next, value)) {
                this.unsubscribe();
            }
        }
    };
    SafeSubscriber.prototype.error = function (err) {
        if (!this.isStopped) {
            var _parentSubscriber = this._parentSubscriber;
            var useDeprecatedSynchronousErrorHandling = config.useDeprecatedSynchronousErrorHandling;
            if (this._error) {
                if (!useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                    this.__tryOrUnsub(this._error, err);
                    this.unsubscribe();
                }
                else {
                    this.__tryOrSetError(_parentSubscriber, this._error, err);
                    this.unsubscribe();
                }
            }
            else if (!_parentSubscriber.syncErrorThrowable) {
                this.unsubscribe();
                if (useDeprecatedSynchronousErrorHandling) {
                    throw err;
                }
                hostReportError(err);
            }
            else {
                if (useDeprecatedSynchronousErrorHandling) {
                    _parentSubscriber.syncErrorValue = err;
                    _parentSubscriber.syncErrorThrown = true;
                }
                else {
                    hostReportError(err);
                }
                this.unsubscribe();
            }
        }
    };
    SafeSubscriber.prototype.complete = function () {
        var _this = this;
        if (!this.isStopped) {
            var _parentSubscriber = this._parentSubscriber;
            if (this._complete) {
                var wrappedComplete = function () { return _this._complete.call(_this._context); };
                if (!config.useDeprecatedSynchronousErrorHandling || !_parentSubscriber.syncErrorThrowable) {
                    this.__tryOrUnsub(wrappedComplete);
                    this.unsubscribe();
                }
                else {
                    this.__tryOrSetError(_parentSubscriber, wrappedComplete);
                    this.unsubscribe();
                }
            }
            else {
                this.unsubscribe();
            }
        }
    };
    SafeSubscriber.prototype.__tryOrUnsub = function (fn, value) {
        try {
            fn.call(this._context, value);
        }
        catch (err) {
            this.unsubscribe();
            if (config.useDeprecatedSynchronousErrorHandling) {
                throw err;
            }
            else {
                hostReportError(err);
            }
        }
    };
    SafeSubscriber.prototype.__tryOrSetError = function (parent, fn, value) {
        if (!config.useDeprecatedSynchronousErrorHandling) {
            throw new Error('bad call');
        }
        try {
            fn.call(this._context, value);
        }
        catch (err) {
            if (config.useDeprecatedSynchronousErrorHandling) {
                parent.syncErrorValue = err;
                parent.syncErrorThrown = true;
                return true;
            }
            else {
                hostReportError(err);
                return true;
            }
        }
        return false;
    };
    SafeSubscriber.prototype._unsubscribe = function () {
        var _parentSubscriber = this._parentSubscriber;
        this._context = null;
        this._parentSubscriber = null;
        _parentSubscriber.unsubscribe();
    };
    return SafeSubscriber;
}(Subscriber));

/** PURE_IMPORTS_START _Subscriber PURE_IMPORTS_END */
function canReportError(observer) {
    while (observer) {
        var _a = observer, closed_1 = _a.closed, destination = _a.destination, isStopped = _a.isStopped;
        if (closed_1 || isStopped) {
            return false;
        }
        else if (destination && destination instanceof Subscriber) {
            observer = destination;
        }
        else {
            observer = null;
        }
    }
    return true;
}

/** PURE_IMPORTS_START _Subscriber,_symbol_rxSubscriber,_Observer PURE_IMPORTS_END */
function toSubscriber(nextOrObserver, error, complete) {
    if (nextOrObserver) {
        if (nextOrObserver instanceof Subscriber) {
            return nextOrObserver;
        }
        if (nextOrObserver[rxSubscriber]) {
            return nextOrObserver[rxSubscriber]();
        }
    }
    if (!nextOrObserver && !error && !complete) {
        return new Subscriber(empty);
    }
    return new Subscriber(nextOrObserver, error, complete);
}

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
var observable = /*@__PURE__*/ (function () { return typeof Symbol === 'function' && Symbol.observable || '@@observable'; })();

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
function identity(x) {
    return x;
}

/** PURE_IMPORTS_START _identity PURE_IMPORTS_END */
function pipeFromArray(fns) {
    if (fns.length === 0) {
        return identity;
    }
    if (fns.length === 1) {
        return fns[0];
    }
    return function piped(input) {
        return fns.reduce(function (prev, fn) { return fn(prev); }, input);
    };
}

/** PURE_IMPORTS_START _util_canReportError,_util_toSubscriber,_symbol_observable,_util_pipe,_config PURE_IMPORTS_END */
var Observable = /*@__PURE__*/ (function () {
    function Observable(subscribe) {
        this._isScalar = false;
        if (subscribe) {
            this._subscribe = subscribe;
        }
    }
    Observable.prototype.lift = function (operator) {
        var observable = new Observable();
        observable.source = this;
        observable.operator = operator;
        return observable;
    };
    Observable.prototype.subscribe = function (observerOrNext, error, complete) {
        var operator = this.operator;
        var sink = toSubscriber(observerOrNext, error, complete);
        if (operator) {
            sink.add(operator.call(sink, this.source));
        }
        else {
            sink.add(this.source || (config.useDeprecatedSynchronousErrorHandling && !sink.syncErrorThrowable) ?
                this._subscribe(sink) :
                this._trySubscribe(sink));
        }
        if (config.useDeprecatedSynchronousErrorHandling) {
            if (sink.syncErrorThrowable) {
                sink.syncErrorThrowable = false;
                if (sink.syncErrorThrown) {
                    throw sink.syncErrorValue;
                }
            }
        }
        return sink;
    };
    Observable.prototype._trySubscribe = function (sink) {
        try {
            return this._subscribe(sink);
        }
        catch (err) {
            if (config.useDeprecatedSynchronousErrorHandling) {
                sink.syncErrorThrown = true;
                sink.syncErrorValue = err;
            }
            if (canReportError(sink)) {
                sink.error(err);
            }
            else {
                console.warn(err);
            }
        }
    };
    Observable.prototype.forEach = function (next, promiseCtor) {
        var _this = this;
        promiseCtor = getPromiseCtor(promiseCtor);
        return new promiseCtor(function (resolve, reject) {
            var subscription;
            subscription = _this.subscribe(function (value) {
                try {
                    next(value);
                }
                catch (err) {
                    reject(err);
                    if (subscription) {
                        subscription.unsubscribe();
                    }
                }
            }, reject, resolve);
        });
    };
    Observable.prototype._subscribe = function (subscriber) {
        var source = this.source;
        return source && source.subscribe(subscriber);
    };
    Observable.prototype[observable] = function () {
        return this;
    };
    Observable.prototype.pipe = function () {
        var operations = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operations[_i] = arguments[_i];
        }
        if (operations.length === 0) {
            return this;
        }
        return pipeFromArray(operations)(this);
    };
    Observable.prototype.toPromise = function (promiseCtor) {
        var _this = this;
        promiseCtor = getPromiseCtor(promiseCtor);
        return new promiseCtor(function (resolve, reject) {
            var value;
            _this.subscribe(function (x) { return value = x; }, function (err) { return reject(err); }, function () { return resolve(value); });
        });
    };
    Observable.create = function (subscribe) {
        return new Observable(subscribe);
    };
    return Observable;
}());
function getPromiseCtor(promiseCtor) {
    if (!promiseCtor) {
        promiseCtor =  Promise;
    }
    if (!promiseCtor) {
        throw new Error('no Promise impl found');
    }
    return promiseCtor;
}

/** PURE_IMPORTS_START  PURE_IMPORTS_END */
var ObjectUnsubscribedErrorImpl = /*@__PURE__*/ (function () {
    function ObjectUnsubscribedErrorImpl() {
        Error.call(this);
        this.message = 'object unsubscribed';
        this.name = 'ObjectUnsubscribedError';
        return this;
    }
    ObjectUnsubscribedErrorImpl.prototype = /*@__PURE__*/ Object.create(Error.prototype);
    return ObjectUnsubscribedErrorImpl;
})();
var ObjectUnsubscribedError = ObjectUnsubscribedErrorImpl;

/** PURE_IMPORTS_START tslib,_Subscription PURE_IMPORTS_END */
var SubjectSubscription = /*@__PURE__*/ (function (_super) {
    __extends(SubjectSubscription, _super);
    function SubjectSubscription(subject, subscriber) {
        var _this = _super.call(this) || this;
        _this.subject = subject;
        _this.subscriber = subscriber;
        _this.closed = false;
        return _this;
    }
    SubjectSubscription.prototype.unsubscribe = function () {
        if (this.closed) {
            return;
        }
        this.closed = true;
        var subject = this.subject;
        var observers = subject.observers;
        this.subject = null;
        if (!observers || observers.length === 0 || subject.isStopped || subject.closed) {
            return;
        }
        var subscriberIndex = observers.indexOf(this.subscriber);
        if (subscriberIndex !== -1) {
            observers.splice(subscriberIndex, 1);
        }
    };
    return SubjectSubscription;
}(Subscription));

/** PURE_IMPORTS_START tslib,_Observable,_Subscriber,_Subscription,_util_ObjectUnsubscribedError,_SubjectSubscription,_internal_symbol_rxSubscriber PURE_IMPORTS_END */
var SubjectSubscriber = /*@__PURE__*/ (function (_super) {
    __extends(SubjectSubscriber, _super);
    function SubjectSubscriber(destination) {
        var _this = _super.call(this, destination) || this;
        _this.destination = destination;
        return _this;
    }
    return SubjectSubscriber;
}(Subscriber));
var Subject = /*@__PURE__*/ (function (_super) {
    __extends(Subject, _super);
    function Subject() {
        var _this = _super.call(this) || this;
        _this.observers = [];
        _this.closed = false;
        _this.isStopped = false;
        _this.hasError = false;
        _this.thrownError = null;
        return _this;
    }
    Subject.prototype[rxSubscriber] = function () {
        return new SubjectSubscriber(this);
    };
    Subject.prototype.lift = function (operator) {
        var subject = new AnonymousSubject(this, this);
        subject.operator = operator;
        return subject;
    };
    Subject.prototype.next = function (value) {
        if (this.closed) {
            throw new ObjectUnsubscribedError();
        }
        if (!this.isStopped) {
            var observers = this.observers;
            var len = observers.length;
            var copy = observers.slice();
            for (var i = 0; i < len; i++) {
                copy[i].next(value);
            }
        }
    };
    Subject.prototype.error = function (err) {
        if (this.closed) {
            throw new ObjectUnsubscribedError();
        }
        this.hasError = true;
        this.thrownError = err;
        this.isStopped = true;
        var observers = this.observers;
        var len = observers.length;
        var copy = observers.slice();
        for (var i = 0; i < len; i++) {
            copy[i].error(err);
        }
        this.observers.length = 0;
    };
    Subject.prototype.complete = function () {
        if (this.closed) {
            throw new ObjectUnsubscribedError();
        }
        this.isStopped = true;
        var observers = this.observers;
        var len = observers.length;
        var copy = observers.slice();
        for (var i = 0; i < len; i++) {
            copy[i].complete();
        }
        this.observers.length = 0;
    };
    Subject.prototype.unsubscribe = function () {
        this.isStopped = true;
        this.closed = true;
        this.observers = null;
    };
    Subject.prototype._trySubscribe = function (subscriber) {
        if (this.closed) {
            throw new ObjectUnsubscribedError();
        }
        else {
            return _super.prototype._trySubscribe.call(this, subscriber);
        }
    };
    Subject.prototype._subscribe = function (subscriber) {
        if (this.closed) {
            throw new ObjectUnsubscribedError();
        }
        else if (this.hasError) {
            subscriber.error(this.thrownError);
            return Subscription.EMPTY;
        }
        else if (this.isStopped) {
            subscriber.complete();
            return Subscription.EMPTY;
        }
        else {
            this.observers.push(subscriber);
            return new SubjectSubscription(this, subscriber);
        }
    };
    Subject.prototype.asObservable = function () {
        var observable = new Observable();
        observable.source = this;
        return observable;
    };
    Subject.create = function (destination, source) {
        return new AnonymousSubject(destination, source);
    };
    return Subject;
}(Observable));
var AnonymousSubject = /*@__PURE__*/ (function (_super) {
    __extends(AnonymousSubject, _super);
    function AnonymousSubject(destination, source) {
        var _this = _super.call(this) || this;
        _this.destination = destination;
        _this.source = source;
        return _this;
    }
    AnonymousSubject.prototype.next = function (value) {
        var destination = this.destination;
        if (destination && destination.next) {
            destination.next(value);
        }
    };
    AnonymousSubject.prototype.error = function (err) {
        var destination = this.destination;
        if (destination && destination.error) {
            this.destination.error(err);
        }
    };
    AnonymousSubject.prototype.complete = function () {
        var destination = this.destination;
        if (destination && destination.complete) {
            this.destination.complete();
        }
    };
    AnonymousSubject.prototype._subscribe = function (subscriber) {
        var source = this.source;
        if (source) {
            return this.source.subscribe(subscriber);
        }
        else {
            return Subscription.EMPTY;
        }
    };
    return AnonymousSubject;
}(Subject));

var isExist = function (obj) {
    if (obj === undefined || obj === null) {
        return false;
    }
    if (Array.isArray(obj)) {
        return obj.length > 0;
    }
    return false;
};
var fromEntries = function (arr) {
    return arr.reduce(function (acc, curr) {
        acc[curr[0]] = curr[1];
        return acc;
    }, {});
};

var methOptFromLogicOpt = function (logicOptions) {
    var requestSequence = logicOptions.requestSequence;
    return {
        requestType: requestSequence.type,
        requestList: __spreadArrays(requestSequence.requestList)
    };
};
var compOptFromLogicOpt = function (logicOptions) {
    var requestVariablesObj = {};
    var computedVariableObj = {};
    var statusVariableObj = {};
    var requestSequence = logicOptions.requestSequence;
    var requestList = requestSequence.requestList;
    requestList
        .forEach(function (req) {
        var _a, _b, _c, _d;
        var varable = getVarableNameFromRequest(req, '');
        if (varable !== '') {
            req.key = (_a = req.key) !== null && _a !== void 0 ? _a : varable;
            req.varable = (_b = req.varable) !== null && _b !== void 0 ? _b : varable;
            req.existVarable = (_c = req.existVarable) !== null && _c !== void 0 ? _c : getExistNameFromVarableName(varable);
            req.statusVarable = (_d = req.statusVarable) !== null && _d !== void 0 ? _d : getStatusNameFromVarableName(varable);
            requestVariablesObj[req.varable] = req.defaultValue;
            computedVariableObj[req.existVarable] = function () {
                var context = this;
                return isExist(context.$data[varable]);
            };
            statusVariableObj[req.statusVarable] = RequestStatus.INIT;
        }
    });
    /**
     * 不允许用户修改
     */
    computedVariableObj['requestStatus'] = function () {
        var context = this;
        return context.$data['_requestStatus'];
    };
    return {
        logicOptions: logicOptions,
        variablesObj: __assign(__assign(__assign({}, requestVariablesObj), fromEntries(requestList.map(function (req) { return ([req.varable + "Status", { req: req, status: RequestStatus.INIT }]); }))), { _requestStatus: requestList.map(function (req) { return ({ req: req, status: RequestStatus.INIT }); }) }),
        computedVariableObj: computedVariableObj
    };
};
var getVarableNameFromRequest = function (request, defaultName) {
    if (defaultName === void 0) { defaultName = null; }
    if (request.varable) {
        return request.varable;
    }
    var found = request.methodName.match(/_(.*)Request/);
    if (found) {
        return "" + found[1];
    }
    return defaultName;
};
var getMethodName = function (req) {
    if (typeof req === 'string') {
        return req;
    }
    else if (typeof req === 'object') {
        return req.methodName;
    }
    else {
        throw new Error('非法 requestList 类型');
    }
};
var getExistNameFromVarableName = function (varableName) {
    if (varableName === '') {
        return null;
    }
    return "has" + lodash.upperFirst(varableName);
};
var getStatusNameFromVarableName = function (varableName) {
    if (varableName === '') {
        return null;
    }
    return varableName + "Status";
};

var BaseMethods = {};
var BaseComponent = /** @class */ (function () {
    function BaseComponent(options, logicOptions) {
        this.options = null;
        this.subject = null;
        this.options = options;
        this.logicOptions = logicOptions;
    }
    BaseComponent.prototype.created = function (context) {
        this.subject = new Subject();
    };
    BaseComponent.prototype.mounted = function (context) {
    };
    BaseComponent.prototype.computed = function (context) {
        var computedVariableObj = this.options.computedVariableObj;
        return __assign({}, computedVariableObj);
    };
    BaseComponent.prototype.data = function (context) {
        var variablesObj = this.options.variablesObj;
        return __assign({}, variablesObj);
    };
    return BaseComponent;
}());
var BaseLogic = /** @class */ (function () {
    function BaseLogic(options) {
        if (options === void 0) { options = {}; }
        this.options = null;
        this.options = deepmerge(Object.getPrototypeOf(this).constructor.defaultOptions, options);
    }
    BaseLogic.prototype.getOptions = function () {
        return this.options;
    };
    BaseLogic.prototype.logAllProperty = function () {
        console.log('子类未实现 打印所有可用属性 方法');
    };
    BaseLogic.prototype.createLogic = function () {
        var _a = Object.getPrototypeOf(this).constructor, Component = _a.Component, Methods = _a.Methods;
        if (!Component || !Methods) {
            throw new Error('子类需要提供 Component 及 Methods 定义');
        }
        var options = this.getOptions();
        var component = new Component({
            logicOptions: options,
            variablesObj: {},
            computedVariableObj: {},
        }, options);
        var methods = createMethodsSet(Methods, {}, component);
        // 退出的对象上下文改变成 vue 上下文
        return {
            created: function () {
                component.created(this);
            },
            mounted: function () {
                component.mounted(this);
            },
            data: function () {
                return component.data(this);
            },
            computed: __assign({}, component.computed(this)),
            methods: __assign({}, methods)
        };
    };
    BaseLogic.makeLogic = function (opt) {
        var Class = Object.getPrototypeOf(this).constructor;
        var logic = new this(opt || Class.defaultOptions);
        return {
            main: logic.createLogic()
        };
    };
    BaseLogic.defaultOptions = { type: LogicType.MIXIN };
    BaseLogic.Component = null;
    BaseLogic.Methods = null;
    return BaseLogic;
}());
var createMethodsSet = function (m, opt, comp, baseMethods) {
    if (baseMethods === void 0) { baseMethods = BaseMethods; }
    var _m = Object.create(baseMethods, {
        options: {
            writable: false,
            configurable: true,
            value: opt
        },
        component: {
            writable: false,
            configurable: true,
            value: comp
        }
    });
    for (var key in m) {
        if (m.hasOwnProperty(key)) {
            _m[key] = m[key];
        }
    }
    var methodObj = {};
    var _loop_1 = function (key) {
        if (_m.hasOwnProperty(key)) {
            methodObj[key] = function () {
                var _a;
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return (_a = _m[key]).call.apply(_a, __spreadArrays([_m, this], args));
            };
        }
    };
    for (var key in _m) {
        _loop_1(key);
    }
    return methodObj;
};

var EMPTY_LOGIC_OPTION = {
    type: LogicType.MIXIN,
    requestListInspect: function (_) { return _; },
    requestSequence: {
        type: RequestSequenceType.CONCURRENCY,
        requestList: [],
    },
};
var updateRequestStatus = function (context, req, status) {
    var index = context.$data['_requestStatus'].findIndex(function (_) {
        return _.req.methodName === req.methodName;
    });
    context.$set(context.$data['_requestStatus'], index, __assign(__assign({}, context.$data['_requestStatus'][index]), { status: status }));
    context.$data[req.statusVarable] = status;
};
var BaseViewMethods = {
    __initData: function (context) {
        this.__fetchRequestList(context);
    },
    __fetchRequest: function (context, requestKey, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        var subject = this.component.subject;
        var requestList = this.options.requestList;
        var req = requestList.find(function (_) { return _.key === requestKey; });
        if (!req) {
            console.warn("\u6CA1\u6709\u627E\u5230 Request Key: " + requestKey);
            return;
        }
        var reqObservable = new Observable(function (subscriber) {
            var methodName = getMethodName(req);
            var id = uuid.v4();
            var nextObj = {
                type: 'request',
                id: id,
                methodName: methodName,
            };
            subscriber.next(nextObj);
            subject.next(nextObj);
            if (!context[methodName]) {
                throw new Error("\u6CA1\u6709\u627E\u5230\u65B9\u6CD5 " + methodName);
            }
            updateRequestStatus(context, req, RequestStatus.HOLD);
            context[methodName]().then(function (data) {
                var nextObj = {
                    type: 'response',
                    id: id,
                    methodName: methodName,
                    data: data,
                };
                _this.__assignResponseData(context, req, data, options, function defaultAssign(context, req, data, options) {
                    var _a = options.isAppend, isAppend = _a === void 0 ? false : _a;
                    var varable = req.varable;
                    if (isAppend) {
                        // TODO: 数组改成 可迭代对象
                        if (!Array.isArray(context[varable])) {
                            throw new Error('非Array对象不可进行追加数据');
                        }
                        context[varable] = context[varable].concat(data);
                    }
                    else {
                        context[varable] = data;
                    }
                });
                updateRequestStatus(context, req, RequestStatus.FINISH);
                subscriber.next(nextObj);
                subject.next(nextObj);
            }, function (error) {
                subscriber.next({
                    type: 'response',
                    id: id,
                    methodName: methodName,
                    error: error,
                });
            });
        });
        reqObservable.subscribe(function (v) {
            if (context.networkListener) {
                context.networkListener.call(context, v);
            }
        });
    },
    __fetchRequestList: function (context, keyList) {
        var _a = this.options, requestList = _a.requestList, requestType = _a.requestType;
        if (requestType === RequestSequenceType.CONCURRENCY) {
            for (var index = 0; index < requestList.length; index++) {
                var req = requestList[index];
                if (Array.isArray(keyList) && keyList.indexOf(req.key) === -1)
                    continue;
                this.__fetchRequest(context, req.key, {});
            }
        }
    },
    __assignResponseData: function (context, req, data, options, defaultAssign) {
        defaultAssign(context, req, data, options);
    }
};
var BaseViewComponent = /** @class */ (function (_super) {
    __extends(BaseViewComponent, _super);
    function BaseViewComponent(options, logicOptions) {
        var _this = _super.call(this, options, logicOptions) || this;
        _this.options = null;
        _this.subject = null;
        _this.options = options;
        _this.logicOptions = logicOptions;
        return _this;
    }
    BaseViewComponent.prototype.created = function (context) {
        this.subject = new Subject();
    };
    BaseViewComponent.prototype.mounted = function (context) {
    };
    BaseViewComponent.prototype.computed = function (context) {
        var computedVariableObj = this.options.computedVariableObj;
        return __assign({}, computedVariableObj);
    };
    BaseViewComponent.prototype.data = function (context) {
        var variablesObj = this.options.variablesObj;
        return __assign({}, variablesObj);
    };
    return BaseViewComponent;
}(BaseComponent));
var BaseViewLogic = /** @class */ (function (_super) {
    __extends(BaseViewLogic, _super);
    function BaseViewLogic(options) {
        return _super.call(this, options) || this;
    }
    BaseViewLogic.prototype.getOptions = function () {
        var options = this.options;
        var requestSequence = options.requestSequence, requestListInspect = options.requestListInspect;
        options.requestSequence = __assign(__assign({}, requestSequence), { requestList: requestListInspect(requestSequence.requestList) });
        return options;
    };
    BaseViewLogic.prototype.logAllProperty = function () {
        console.log('子类未实现 打印所有可用属性 方法');
    };
    BaseViewLogic.prototype.createLogic = function () {
        var _a = Object.getPrototypeOf(this).constructor, Component = _a.Component, Methods = _a.Methods;
        if (!Component || !Methods) {
            throw new Error('子类需要提供 Component 及 Methods 定义');
        }
        var options = this.getOptions();
        var component = new Component(compOptFromLogicOpt(options), options);
        var methods = createMethodsSet(Methods, methOptFromLogicOpt(options), component, BaseViewMethods);
        // 退出的对象上下文改变成 vue 上下文
        return {
            created: function () {
                component.created(this);
            },
            mounted: function () {
                component.mounted(this);
            },
            data: function () {
                return component.data(this);
            },
            computed: __assign({}, component.computed(this)),
            methods: __assign({}, methods)
        };
    };
    BaseViewLogic.makeLogic = function (opt) {
        var Class = Object.getPrototypeOf(this).constructor;
        var logic = typeof opt === 'function' ? new this(opt(Class.defaultOptions))
            : new this(opt || Class.defaultOptions || EMPTY_LOGIC_OPTION);
        return {
            main: logic.createLogic()
        };
    };
    BaseViewLogic.defaultOptions = null;
    BaseViewLogic.Component = null;
    BaseViewLogic.Methods = null;
    return BaseViewLogic;
}(BaseLogic));

var DEFAULT_OPTIONS = {
    type: LogicType.MIXIN,
    requestListInspect: function (_) { return _; },
    requestSequence: {
        type: RequestSequenceType.CONCURRENCY,
        requestList: [
            {
                methodName: '_bannersRequest',
                defaultValue: [],
                varable: 'banners'
            }, {
                methodName: '_hotFilmsRequest',
                defaultValue: [],
            }, {
                methodName: '_waitFilmsRequest',
                defaultValue: []
            }, {
                methodName: '_cinemasRequest',
                defaultValue: [],
            },
        ],
    },
};
var Methods = {
    loadMoreCinema: function (context) {
        this.__fetchRequest(context, 'cinemas', { isAppend: true });
    },
    reloadData: function (context, keyList, options) {
        if (keyList === undefined) {
            this.__fetchRequestList(context);
        }
        else {
            this.__fetchRequestList(context, keyList);
        }
    },
    _initData: function (context) {
        this.__fetchRequestList(context);
    },
    _bannersRequest: function (context) {
        return __awaiter(this, void 0, void 0, function () {
            var NetworkBanner;
            return __generator(this, function (_a) {
                NetworkBanner = getNetwork('NetworkBanner');
                return [2 /*return*/, NetworkBanner.useParams('cityId').bannerList()];
            });
        });
    },
    _hotFilmsRequest: function (context) {
        return __awaiter(this, void 0, void 0, function () {
            var NetworkFilm;
            return __generator(this, function (_a) {
                NetworkFilm = getNetwork('NetworkFilm');
                return [2 /*return*/, NetworkFilm.useParams('cityId').hotFilms({
                        page: 1,
                        size: 20
                    })];
            });
        });
    },
    _waitFilmsRequest: function (context) {
        return __awaiter(this, void 0, void 0, function () {
            var NetworkFilm;
            return __generator(this, function (_a) {
                NetworkFilm = getNetwork('NetworkFilm');
                return [2 /*return*/, NetworkFilm.useParams('cityId').waitFilms({
                        page: 1,
                        size: 20
                    })];
            });
        });
    },
    _cinemasRequest: function (context) {
        return __awaiter(this, void 0, void 0, function () {
            var NetworkCinema;
            return __generator(this, function (_a) {
                NetworkCinema = getNetwork('NetworkCinema');
                return [2 /*return*/, NetworkCinema.useParams('location').cinemasList({
                        page: context.cinemasPage,
                        size: context.cinemasSize,
                    }).then(function (data) {
                        context.cinemasPage++;
                        return data;
                    })];
            });
        });
    },
};
var Component = /** @class */ (function (_super) {
    __extends(Component, _super);
    function Component() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Component.prototype.mounted = function (context) {
        _super.prototype.mounted.call(this, context);
        context._initData();
    };
    Component.prototype.data = function (context) {
        return __assign(__assign({}, _super.prototype.data.call(this, context)), { cinemasPage: 1, cinemasSize: 10 });
    };
    return Component;
}(BaseViewComponent));
/**
 * 首页 logic 类
 */
var HomeViewLogic = /** @class */ (function (_super) {
    __extends(HomeViewLogic, _super);
    function HomeViewLogic(opt) {
        return _super.call(this, opt) || this;
    }
    HomeViewLogic.defaultOptions = DEFAULT_OPTIONS;
    HomeViewLogic.Component = Component;
    HomeViewLogic.Methods = Methods;
    return HomeViewLogic;
}(BaseViewLogic));

var DEFAULT_OPTIONS$1 = {
    type: LogicType.MIXIN,
    requestListInspect: function (_) { return _; },
    requestSequence: {
        type: RequestSequenceType.CONCURRENCY,
        requestList: [
            {
                methodName: '_cinemasRequest',
                defaultValue: [],
            },
            {
                methodName: '_districtsRequest',
                defaultValue: [],
            },
            {
                methodName: '_bannersRequest',
                defaultValue: [],
            }
        ],
    },
};
var ORDER_TYPE;
(function (ORDER_TYPE) {
    ORDER_TYPE[ORDER_TYPE["DISTANCE"] = 1] = "DISTANCE";
    ORDER_TYPE[ORDER_TYPE["PRICE"] = 2] = "PRICE";
})(ORDER_TYPE || (ORDER_TYPE = {}));
var PAGE_SIZE = 20;
var Methods$1 = {
    reloadCinemas: function (context) {
        window.scrollTo(0, 1);
        context['page'] = 1;
        context['finishLoading'] = false;
        this.__fetchRequest(context, 'cinemas');
    },
    loadMoreCinemas: function (context) {
        if (context['cinemasStatus'] === 'hold' || context['finishLoading']) {
            console.log('加载完成');
        }
        context['page']++;
        this.__fetchRequest(context, 'cinemas', { isAppend: true });
    },
    _initData: function (context) {
        this.__fetchRequestList(context);
    },
    _cinemasRequest: function (context) {
        return __awaiter(this, void 0, void 0, function () {
            var region, orderType, feature, page, _a, size, regionName;
            return __generator(this, function (_b) {
                region = context.region, orderType = context.orderType, feature = context.feature, page = context.page, _a = context.size, size = _a === void 0 ? PAGE_SIZE : _a;
                regionName = (region === null || region === void 0 ? void 0 : region.name) === '全部' ? '' : region === null || region === void 0 ? void 0 : region.name;
                return [2 /*return*/, getNetwork('NetworkCinema').useParams('location').cinemasList({
                        regionName: regionName,
                        orderType: orderType,
                        feature: feature,
                        page: page,
                        size: size
                    }).then(function (data) {
                        context.finishLoading = data.length < PAGE_SIZE;
                        return data;
                    })];
            });
        });
    },
    _districtsRequest: function (context) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, getNetwork('NetworkCity').useParams('cityId').cityDistrictList()];
            });
        });
    },
    _bannersRequest: function (context) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, getNetwork('NetworkBanner').useParams('cityId').bannerList('02')];
            });
        });
    },
    __assignResponseData: function (context, req, data, options, defaultAssign) {
        if (req.key === 'districts') {
            context.districts = [{ id: 0, name: '全部' }].concat(data);
            return;
        }
        defaultAssign(context, req, data, options);
    }
};
var Component$1 = /** @class */ (function (_super) {
    __extends(Component, _super);
    function Component() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Component.prototype.mounted = function (context) {
        context._initData();
    };
    Component.prototype.data = function (context) {
        return __assign(__assign({}, _super.prototype.data.call(this, context)), { region: null, orderType: context.hasLocation ? ORDER_TYPE.DISTANCE : ORDER_TYPE.PRICE, feature: null, page: 1, finishLoading: false });
    };
    return Component;
}(BaseViewComponent));
var CinemaListLogic = /** @class */ (function (_super) {
    __extends(CinemaListLogic, _super);
    function CinemaListLogic(opt) {
        return _super.call(this, opt) || this;
    }
    CinemaListLogic.defaultOptions = DEFAULT_OPTIONS$1;
    CinemaListLogic.Component = Component$1;
    CinemaListLogic.Methods = Methods$1;
    return CinemaListLogic;
}(BaseViewLogic));

var CoordinateNull = {
    longitude: null,
    latitude: null,
};
var instance = null;
var LocationManager = /** @class */ (function () {
    function LocationManager(options) {
        var _this = this;
        this.options = null;
        if (!instance) {
            instance = this;
            this.options = options = __assign(__assign({}, getMapConfig()), options);
            options.nativeLocationCallback(function (location) {
                _this.locationCallback(location);
            });
            this.nativeLocationCallback = null;
        }
        return instance;
    }
    LocationManager.prepareManager = function (options) {
        if (options === void 0) { options = {}; }
        if (instance) {
            throw new Error('只能执行 prepareManager 一次');
        }
        return new LocationManager(options);
    };
    LocationManager.defaultManager = function () {
        if (!instance) {
            throw new Error('需要执行 prepareManager');
        }
        return instance;
    };
    LocationManager.prototype.locationCallback = function (location) {
        if (this.nativeLocationCallback) {
            this.nativeLocationCallback(location);
        }
    };
    /**
     * app定位超时处理
     */
    LocationManager.prototype.appLocationTimeOutPromise = function (timeout) {
        if (timeout === void 0) { timeout = 1500; }
        return __awaiter(this, void 0, Promise, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve) {
                        setTimeout(function () {
                            resolve({ longitude: null, latitude: null });
                        }, timeout);
                    })];
            });
        });
    };
    LocationManager.prototype.getNativeLocation = function () {
        var _this = this;
        var nativeLocation = this.options.nativeLocation;
        return new Promise(function (resolve, reject) {
            _this.nativeLocationCallback = function (location) {
                resolve(location);
            };
            try {
                nativeLocation();
            }
            catch (e) {
                resolve(null);
            }
        });
    };
    LocationManager.prototype.nativeLocationNull = function (coordinate) {
        var nativeLocationIsNull = this.options.nativeLocationIsNull;
        if (nativeLocationIsNull) {
            return nativeLocationIsNull(coordinate);
        }
        return !!coordinate;
    };
    /**
     * 获取定位（经纬度）
     * @returns {Promise}
     */
    LocationManager.prototype.getLocation = function (locatorList, options) {
        var _a;
        return __awaiter(this, void 0, void 0, function () {
            var openNativeLocation, timeout, i, l, locator, coordinate;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        openNativeLocation = false, timeout = options.timeout;
                        for (i = 0, l = locatorList.length; i < l; i++) {
                            locator = locatorList[i];
                            if (locator.key === LocatorType.NATIVE) {
                                openNativeLocation = true;
                                timeout = (_a = locator.timeout) !== null && _a !== void 0 ? _a : timeout;
                            }
                        }
                        coordinate = CoordinateNull;
                        if (!openNativeLocation) return [3 /*break*/, 2];
                        return [4 /*yield*/, Promise.race([this.appLocationTimeOutPromise(timeout), this.getNativeLocation()]).then(function (coordinate) {
                                var longitude = null;
                                var latitude = null;
                                if (!_this.nativeLocationNull(coordinate)) {
                                    longitude = coordinate.longitude;
                                    latitude = coordinate.latitude;
                                }
                                return {
                                    longitude: longitude,
                                    latitude: latitude,
                                };
                            }).catch(function (error) {
                                console.log(error);
                                return CoordinateNull;
                            })];
                    case 1:
                        coordinate = _b.sent();
                        _b.label = 2;
                    case 2: return [2 /*return*/, new Promise(function (resolve, reject) {
                            var longitude = coordinate.longitude, latitude = coordinate.latitude;
                            _this.handleLocationPromise(locatorList.filter(function (_a) {
                                var key = _a.key;
                                return key !== LocatorType.NATIVE;
                            }), longitude, latitude).then(function (location) {
                                // this.updateLocationStore(location)
                                resolve(location);
                            }, function (error) {
                                // this.updateLocationStore()
                                reject(error);
                            });
                        })];
                }
            });
        });
    };
    /**
     * web定位处理回调
     * @param longitude 定位到得经纬度
     * @param latitude 定位到的经纬度
     */
    LocationManager.prototype.handleLocationPromise = function (locatorList, longitude, latitude) {
        if (longitude && latitude) {
            return new Promise(function (resolve, reject) {
                console.log('good, the native app location is enable');
                resolve({ longitude: longitude, latitude: latitude });
            });
        }
        else {
            var _a = this.options, mapPlatforms = _a.mapPlatforms, mapFrameworks = _a.mapFrameworks;
            return new Promise(function (resolve, reject) {
                console.log('oops, the native app location is fly');
                // locatorList.map(({key}) => {
                //   const index = mapPlatforms.findIndex(*)
                // })
                // Promise.any([this.baiduLocatePromise()]).then(data => {
                //   console.log(data);
                // });
                // this.baiduLocatePromise().then(location => {
                //   resolve(location)
                // }, error => {
                //   reject(error)
                // })
            });
        }
    };
    /**
     * 浏览器定位
     */
    LocationManager.prototype.browserLocate = function () {
        return new Promise(function (resolve, reject) {
            // navigator.geolocation.getCurrentPosition(location => {
            //   this.doSuccessCallback(location)
            // }, () => {
            //   this.doErrorCallback()
            // })
        });
    };
    /**
     * 百度定位
     */
    LocationManager.prototype.baiduLocatePromise = function (BMap) {
        return new Promise(function (resolve, reject) {
            var geolocation = new BMap.Geolocation();
            geolocation.getCurrentPosition(function (options) {
                if (geolocation.getStatus() === 0 && options.accuracy !== null) {
                    resolve(options);
                }
                else {
                    reject();
                }
            }, function () {
                reject();
            }, { timeout: 2000, maximumAge: 0 });
        });
    };
    return LocationManager;
}());

var Methods$2 = {
    _initData: function (context) {
    },
};
var Component$2 = /** @class */ (function (_super) {
    __extends(Component, _super);
    function Component() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Component.prototype.mounted = function (context) {
        var _a = this.logicOptions.locatorSequence, timeout = _a.timeout, locatorList = _a.locatorList;
        LocationManager.prepareManager();
        var manager = LocationManager.defaultManager();
        LocationManager.defaultManager().getLocation(locatorList, { timeout: timeout }).then(function (data) {
            console.log(data);
        });
    };
    Component.prototype.data = function (context) {
        return {};
    };
    return Component;
}(BaseComponent));
var DEFAULT_OPTIONS$2 = {
    type: LogicType.MIXIN,
    locatorSequence: {
        timeout: 1000,
        locatorList: [{
                key: LocatorType.NATIVE,
            }, {
                key: LocatorType.BAIDU,
            }]
    },
};
var LocationLogic = /** @class */ (function (_super) {
    __extends(LocationLogic, _super);
    function LocationLogic(opt) {
        return _super.call(this, opt) || this;
    }
    LocationLogic.makeLogic = function (opt) {
        var logic = new this(opt);
        return {
            main: logic.createLogic()
        };
    };
    LocationLogic.defaultOptions = DEFAULT_OPTIONS$2;
    LocationLogic.Component = Component$2;
    LocationLogic.Methods = Methods$2;
    return LocationLogic;
}(BaseLogic));

var VIEW_LOGIC = {
    HomeView: HomeViewLogic,
    CinemaListView: CinemaListLogic
};
var install = function (vue, config) {
    if (config === void 0) { config = {}; }
    var networkConfig = config.networkConfig, geoConfig = config.geoConfig;
    window.$jbzLogicConfig = vue.$jbzLogicConfig = vue.prototype.$jbzLogicConfig = __assign({}, config);
};
// if (typeof window !== 'undefined' && window.Vue) {
//     install(window.Vue);
// }
function generateLogic(logicName, opt) {
    if (VIEW_LOGIC.hasOwnProperty(logicName)) {
        return VIEW_LOGIC[logicName].makeLogic(opt);
    }
    console.error("\u6CA1\u6709\u627E\u5230Logic\u7C7B: " + logicName);
}
var index = {
    install: install,
    version: version,
};

exports.LocationLogic = LocationLogic;
exports.default = index;
exports.generateLogic = generateLogic;
//# sourceMappingURL=index.js.map
