import {BaseViewComponent, BaseViewLogic} from "../base-view";
import {IViewLogicOptions, IMethods} from "../../interface/IOptions";
import {LogicType, RequestSequenceType} from "../../variable/enum";
import {getNetwork} from '../../util/ConfigManager'

const DEFAULT_OPTIONS: IViewLogicOptions = {
    type: LogicType.MIXIN,
    requestListInspect: _ => _,
    requestSequence: {
        type: RequestSequenceType.CONCURRENCY,
        requestList: [
            {
                methodName: '_cinemasRequest',
                defaultValue: [],
            },
            {
                methodName: '_districtsRequest',
                defaultValue: [],
            },
            {
                methodName: '_bannersRequest',
                defaultValue: [],
            }
        ],
    },
};

enum ORDER_TYPE {
    DISTANCE = 1,
    PRICE
}

const PAGE_SIZE = 20;

const Methods: IMethods<Component> = {
    reloadCinemas(context){
        window.scrollTo(0, 1);
        context['page'] = 1;
        context['finishLoading'] = false;
        this.__fetchRequest(context, 'cinemas')
    },
    loadMoreCinemas(context){
        if (context['cinemasStatus'] === 'hold' || context['finishLoading']){
            console.log('加载完成');
        }
        context['page']++;
        this.__fetchRequest(context, 'cinemas', {isAppend: true})
    },
    _initData(context) {
        this.__fetchRequestList(context);
    },
    async _cinemasRequest(context){
        const {
            region,
            orderType,
            feature,
            page,
            size = PAGE_SIZE,
        } = context;
        const regionName = region?.name === '全部' ? '' : region?.name;
        return getNetwork('NetworkCinema').useParams('location').cinemasList({
            regionName,
            orderType,
            feature,
            page,
            size
        }).then(data => {
            context.finishLoading = data.length < PAGE_SIZE;
            return data
        })
    },
    async _districtsRequest(context){
        return getNetwork('NetworkCity').useParams('cityId').cityDistrictList()
    },
    async _bannersRequest(context){
      return getNetwork('NetworkBanner').useParams('cityId').bannerList('02');
    },
    __assignResponseData(context, req, data, options, defaultAssign){
        if (req.key === 'districts'){
            context.districts = [{id: 0, name: '全部'}].concat(data);
            return;
        }
        defaultAssign(context, req, data, options)
    }
};

class Component extends BaseViewComponent{
    public mounted(context) {
        context._initData();
    }

    public data(context): Object {
        return {
            ...super.data(context),
            region: null,
            orderType: context.hasLocation ? ORDER_TYPE.DISTANCE : ORDER_TYPE.PRICE,
            feature: null,
            page: 1,
            finishLoading: false,
        };
    }
}

class CinemaListLogic extends BaseViewLogic{
    static defaultOptions: IViewLogicOptions = DEFAULT_OPTIONS;
    static Component = Component;
    static Methods = Methods;
    constructor(opt) {
        super(opt);
    }
}

export default CinemaListLogic;
