import {getNetwork} from '../../util/ConfigManager';
import {LogicType, RequestSequenceType} from "../../variable/enum";
import {IViewLogicOptions, IMethods} from "../../interface/IOptions";
import {BaseViewComponent, BaseViewLogic} from "../base-view";

const DEFAULT_OPTIONS: IViewLogicOptions = {
    type: LogicType.MIXIN,
    requestListInspect: _ => _,
    requestSequence: {
        type: RequestSequenceType.CONCURRENCY,
        requestList: [
            {
                methodName: '_bannersRequest',
                defaultValue: [],
                varable: 'banners'
            }, {
                methodName: '_hotFilmsRequest',
                defaultValue: [],
            }, {
                methodName: '_waitFilmsRequest',
                defaultValue: []
            }, {
                methodName: '_cinemasRequest',
                defaultValue: [],
            },
        ],
    },
};

const Methods: IMethods<Component> = {
    loadMoreCinema(context){
        this.__fetchRequest(context, 'cinemas', {isAppend: true})
    },
    reloadData(context, keyList: undefined | string[], options = {}){
        if (keyList === undefined){
            this.__fetchRequestList(context);
        } else {
            this.__fetchRequestList(context, keyList);
        }
    },
    _initData(context){
        this.__fetchRequestList(context);
    },
    async _bannersRequest(context){
        const NetworkBanner = getNetwork('NetworkBanner');
        return NetworkBanner.useParams('cityId').bannerList()
    },
    async _hotFilmsRequest(context){
        const NetworkFilm = getNetwork('NetworkFilm');
        return NetworkFilm.useParams('cityId').hotFilms({
            page: 1,
            size: 20
        })
    },
    async _waitFilmsRequest(context){
        const NetworkFilm = getNetwork('NetworkFilm');
        return NetworkFilm.useParams('cityId').waitFilms({
            page: 1,
            size: 20
        })
    },
    async _cinemasRequest(context){
        const NetworkCinema = getNetwork('NetworkCinema');
        return NetworkCinema.useParams('location').cinemasList({
            page: context.cinemasPage,
            size: context.cinemasSize,
        }).then(data => {
            context.cinemasPage++;
            return data;
        });
    },
};

class Component extends BaseViewComponent{
    public mounted(context) {
        super.mounted(context);
        context._initData();
    }

    public data(context){
        return {
            ...super.data(context),
            cinemasPage: 1,
            cinemasSize: 10,
        }
    }
}

/**
 * 首页 logic 类
 */
class HomeViewLogic extends BaseViewLogic{
    static defaultOptions: IViewLogicOptions = DEFAULT_OPTIONS;
    static Component = Component;
    static Methods = Methods;
    constructor(opt) {
        super(opt);
    }
}

export default HomeViewLogic;
