import {IComponentOptions, IViewLogicApplyOptions, IMethodsOptions} from "@/interface/IOptions";
import {fromEntries, isExist} from "./ObjectTool";
import RequestModel from "../model/RequestModel";
import {upperFirst} from "lodash";
import {RequestStatus} from "@/variable/enum";

const getUnderlineMethods = (obj): Map<string, Function> => {
    const map = new Map();
    for (let key in obj){
        if (obj.hasOwnProperty(key)){
            if (/^_.*$/.test(key) && typeof obj[key] === 'function'){
                map.set(key, obj[key]);
            }
        }
    }
    return map;
};

function getMethods(obj): Map<string, Function> {
    const map = new Map();
    for (let key in obj){
        if (obj.hasOwnProperty(key)){
            map.set(key, obj[key]);
        }
    }
    return map;
}

const methOptFromLogicOpt = (logicOptions: IViewLogicApplyOptions): IMethodsOptions => {
    const {requestSequence} = logicOptions;
    return {
        requestType: requestSequence.type,
        requestList: [...requestSequence.requestList]
    }
};

const compOptFromLogicOpt = (logicOptions: IViewLogicApplyOptions): IComponentOptions => {
    const requestVariablesObj = {};
    const computedVariableObj = {};
    const statusVariableObj = {};
    let {requestSequence} = logicOptions;
    const {requestList} = requestSequence;
    requestList
        .forEach(req => {
            const varable = getVarableNameFromRequest(req, '');
            if (varable !== ''){
                req.key = req.key ?? varable;
                req.varable = req.varable ?? varable;
                req.existVarable = req.existVarable ?? getExistNameFromVarableName(varable);
                req.statusVarable = req.statusVarable ?? getStatusNameFromVarableName(varable);
                requestVariablesObj[req.varable] = req.defaultValue;
                computedVariableObj[req.existVarable] = function() {
                    const context = this;
                    return isExist(context.$data[varable]);
                };
                statusVariableObj[req.statusVarable] = RequestStatus.INIT;
            }
        });

    /**
     * 不允许用户修改
     */
    computedVariableObj['requestStatus'] = function () {
        const context = this;
        return context.$data['_requestStatus'];
    };

    return {
        logicOptions: logicOptions,
        variablesObj:{
            ...requestVariablesObj,
            ...fromEntries(requestList.map(req => ([`${req.varable}Status`, {req, status: RequestStatus.INIT}]))),
            _requestStatus: requestList.map(req => ({req: req, status: RequestStatus.INIT})),
        },
        computedVariableObj
    }
};

const getVarableNameFromRequest = (request: RequestModel, defaultName: string = null) => {
    if (request.varable){
        return request.varable;
    }
    const found = request.methodName.match(/_(.*)Request/);
    if (found){
        return `${found[1]}`
    }
    return defaultName;
};

const getMethodName = (req: RequestModel | string) => {
    if (typeof req === 'string'){
        return req
    } else if (typeof req === 'object'){
        return req.methodName;
    } else {
        throw new Error('非法 requestList 类型');
    }
};

const getExistNameFromVarableName = (varableName: string): string | null => {
    if (varableName === ''){
        return null;
    }
    return `has${upperFirst(varableName)}`
};

const getStatusNameFromVarableName = (varableName: string): string | null => {
    if (varableName === ''){
        return null;
    }
    return `${varableName}Status`
};

export {
    getUnderlineMethods,
    getMethods,
    methOptFromLogicOpt,
    compOptFromLogicOpt,
    getVarableNameFromRequest,
    getMethodName,
    getExistNameFromVarableName,
}
